@@ -33,18 +33,18 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef TORRENT_VERSION_HPP_INCLUDED
 #define TORRENT_VERSION_HPP_INCLUDED
 
-#include "libtorrent/export.hpp"
+#include "libtorrent/aux_/export.hpp"
 
 #define LIBTORRENT_VERSION_MAJOR 1
-#define LIBTORRENT_VERSION_MINOR 1
-#define LIBTORRENT_VERSION_TINY 7
+#define LIBTORRENT_VERSION_MINOR 2
+#define LIBTORRENT_VERSION_TINY 6
 
 // the format of this version is: MMmmtt
 // M = Major version, m = minor version, t = tiny version
 #define LIBTORRENT_VERSION_NUM ((LIBTORRENT_VERSION_MAJOR * 10000) + (LIBTORRENT_VERSION_MINOR * 100) + LIBTORRENT_VERSION_TINY)
 
-#define LIBTORRENT_VERSION "1.1.7.0"
-#define LIBTORRENT_REVISION "c5a5e084d"
+#define LIBTORRENT_VERSION "1.2.6.0"
+#define LIBTORRENT_REVISION "ac4dd411c"
 
 namespace libtorrent {
 
@@ -54,4 +54,6 @@ namespace libtorrent {
 
 }
 
+namespace lt = libtorrent;
+
 #endif
